#
# tonleiter.py - liefert Daten zu Tonleitern
# W. Arnhold 09.05.2000
# letzte Aenderung 09.05.2000

import math
import string

tonnamen = ['c', 'cis', 'd', 'es', 'e', 'f', 'fis', 'g', 'as', 'a', 'b', 'h']
enharmonische_verwechslung = {
	'cisis': 'd', 'dis': 'es', 'esis': 'e', 'eis': 'f', 'fisis': 'g', 
	'gis': 'as', 'asis': 'a', 'ais': 'b', 'bis': 'h', 'his': 'c',
	'ces': 'h', 'cises': 'c', 'des': 'cis', 'eses': 'd', 'fes': 'e', 
	'fises': 'f', 'ges': 'fis', 'ases': 'g', 'bes': 'a', 'hes': 'b'
}

kammerton = 440
frequenzliste_temperiert = []
frequenzdict_temperiert = {}


MM_wert = 100
tonlaengen = {		# Wieviel Vierundsechzigstel 
	'0': 64,	# ganze Note
	'2': 32,	# halbe Note
	'4': 16,	# Viertelnote
	'8': 8,		# Achtelnote
	'1': 4,		# Sechzehntelnote
	'3': 2,		# 32-stelnote
	'6': 1		# 64-stelnote
}
absolutzchn = '#'	# Tonlaenge als Absolutwert in 64-stel
pausenzchn  = 'P'

tontrennung = 1.0	# Wieviel einer 64-stel werden am Ende einer nicht
			# gebundenen nicht mehr gespielt?

def frequenzinit():
	f = math.pow(2.0, 1.0 / 12.0)
	frequenzfaktoren = []
	start = kammerton / 16.0	# A2
	frequenzliste_temperiert.append(('A2', 27.5))
	i = 0
	frequenzliste_temperiert.append(('B2', frequenzliste_temperiert[i][1] * f))
	i = i + 1
	frequenzliste_temperiert.append(('H2', frequenzliste_temperiert[i][1] * f))
	i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((string.capitalize(name) + '1',
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((string.capitalize(name),
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((name,
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((name + '1',
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((name + '2',
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((name + '3',
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	for name in tonnamen:
		frequenzliste_temperiert.append((name + '4',
			frequenzliste_temperiert[i][1] * f))
		i = i + 1
	frequenzliste_temperiert.append((tonnamen[0] + '5',
		frequenzliste_temperiert[i][1] * f))
	for fr in frequenzliste_temperiert:
		frequenzdict_temperiert[fr[0]] = fr[1]

def standardtonname(name):
#	print name, len(name)
	neuname = name
	anhangziffer = ''
	if name[-1] in string.digits:
		anhangziffer = name[-1]
		neuname = string.lower(name[:-1])
	grossbuchstabe = name[0] in string.uppercase
	if enharmonische_verwechslung.has_key(string.lower(neuname)):
#		print 'gefunden:', neuname
		neuname = enharmonische_verwechslung[neuname]
	else:
		return name
	if grossbuchstabe:
		string.capitalize(neuname)
	return neuname + anhangziffer

def tondauer(wert, mit_trennung = 1):	# Ein: Anzahl der 64-stel, Aus: Sekunden
					# Trennung = ohne Legato
	if mit_trennung:
		abzug = tontrennung
	else:
		abzug = 0.0
	return (wert - abzug) * 3.75 / MM_wert

def phys_ton(s):	# Ermittelt aus einem String die physikalischen
			# Tondaten (dauer(in s), frequenz(in Hz))
			# bzw. (dauer (in s), -1) fuer Pause
	if not s or len(s) < 2:
		return None
	print 's =', s
	if s[0] == absolutzchn:
		ziffern = ''
		for z in s[1:]:
			if not z in string.digits:
				break
			ziffern = ziffern + z
		if not ziffern or len(s) == 1 + len(ziffern):
			return None
		dauer = eval(ziffern)
		s = s[1 + len(ziffern):]
	elif s[0] in string.digits:
		if tonlaengen.has_key(s[0]):
			dauer = tonlaengen[s[0]]
			s = s[1:]
			print dauer, s
		else:
			return None
	else:
		return None
	s = standardtonname(s)
	if s == pausenzchn:
		fr = -1
		return (tondauer(dauer, 0), fr)
	elif not frequenzdict_temperiert.has_key(s):
		print 'Schluessel >>%s<< nicht gefunden' % s
		print 
		return None
	else:
		fr = frequenzdict_temperiert[s]
	print 'Dauer', dauer, 'fr', fr
	return (tondauer(dauer), fr)
	
def MM_setzen(wert):
	global MM_wert
	MM_wert = wert
	
def test():
	frequenzinit()
#	for fr in frequenzliste_temperiert:
#		print fr
	while 1:
		name = raw_input()
		if not name:
			break
#		name = standardtonname(name)
#		if not frequenzdict_temperiert.has_key(name):
#			print 'Einen solchen Ton gibt es nicht!'
#			continue
#		print standardtonname(name), ':', \
#			frequenzdict_temperiert[standardtonname(name)]
		print phys_ton(name)

# test()
frequenzinit()
