# -*- coding: iso-8859-15 -*-
"""\
Datei:		ttest.py
Zweck:		Testen der Tonklasse
Autor:		W. Arnhold
Erstellt:	07.11.2003
Stand:		07.11.2003
"""
from random import Random, seed
import math
from ton import Ton, Klang, Huellkurve, stille, lautstaerke_1
from tonleiter import *

z = 2.0
f = 880.0
r = 22050
samples_zyklus = r/f

def test1():
	t = Ton(dauer = z, frequenz = f, rate = r, lautstaerke = 1.0)
	t.wave_speichern('ton1.wav')
#	plotdat = open('test1.gp', 'w')
#	t.plot(plotdat, normiert = 1, zeitindex = 1)
#	plotdat.close()

# Zeitwert als Parameter
def saegezahn(x):
	return 1.0 - 2.0 * x

def rechteckton(x):
	if x < 0.5:
		return 1
	return -1
	
def test2():
	t = Ton(dauer = 2.0, frequenz = f, rate = r, 
		tonfunktion = saegezahn)
	t.wave_speichern('ton2.wav')

def test3a():
	d = 1.0
	f = 110.0
	t1 = Ton(dauer = d, frequenz = f)
	f = f * 2
	t2 = Ton(dauer = d, frequenz = f)
	f = f * 3 / 2
	t3 = Ton(dauer = d, frequenz = f)
	f = f * 4 / 3
	t4 = Ton(dauer = d, frequenz = f)
	f = f * 5 / 4
	t5 = Ton(dauer = d, frequenz = f)
	f = f * 6 / 5
	t6 = Ton(dauer = d, frequenz = f)
	f = f * 7 / 6
	t7 = Ton(dauer = d, frequenz = f)
	t = Ton()
	t = t1 + t2 + t3 + t4 + t5 + t6 + t7
	print 't:',t.parameter()
	t.wave_speichern('ton3a.wav')
	
def test3b():
	f = frequenzdict_temperiert['c1']
	t1 = Ton(dauer = 0.5, frequenz = f, rate = r)
	f = frequenzdict_temperiert['d1']
	t2 = Ton(dauer = 0.5, frequenz = f, rate = r)
	f = frequenzdict_temperiert['e1']
	t3 = Ton(dauer = 0.5, frequenz = f, rate = r)
	f = frequenzdict_temperiert['f1']
	t4 = Ton(dauer = 0.5, frequenz = f, rate = r)
	f = frequenzdict_temperiert['g1']
	t5 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t = t1 + t2 + t3 + t4 + t5 + t5 + t5 + t5
	t.wave_speichern('ton3b.wav')

def att(x):
	return math.sin(x * math.pi / 2)

def dec(x):
	if x == 0.0:
		return 1.0
	return 0.8 + 0.2* (1 - x)

def sus(x):
	return 0.8

def rel(x):
	return 0.8 - 0.8 ** (1.0/x)

def test3c():
	h = Huellkurve(0.5, (1.0/48, att), (1.0/48, dec), (1.0/48, sus),
		(45.0/48, rel))
	f = frequenzdict_temperiert['c1']
	t1 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t1.einhuellen(h)
	f = frequenzdict_temperiert['d1']
	t2 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t2.einhuellen(h)
	f = frequenzdict_temperiert['e1']
	t3 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t3.einhuellen(h)
	f = frequenzdict_temperiert['f1']
	t4 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t4.einhuellen(h)
	f = frequenzdict_temperiert['g1']
	t5 = Ton(dauer = 0.5, frequenz = f, rate = r)
	t5.einhuellen(h)
	t = t1 + t2 + t3 + t4 + t5 + t5 + t5 + t5
	t.wave_speichern('ton3c.wav')
	plotdat = open('ton3c.gp', 'w')
	t.plot(plotdat, normiert = 1)
	plotdat.close()

def test4():
	f = frequenzdict_temperiert['c1']
	t1 = Ton(dauer = 3.0, frequenz = f, rate = r)
	f = frequenzdict_temperiert['e1']
	t2 = Ton(dauer = 2.5, frequenz = f, rate = r)
	f = frequenzdict_temperiert['g1']
	t3 = Ton(dauer = 2.0, frequenz = f, rate = r)
	f = frequenzdict_temperiert['c2']
	t4 = Ton(dauer = 1.5, frequenz = f, rate = r)
	t1.mischen([(0.5, t2), (1.0, t3), (1.5, t4)])
	t1.wave_speichern('ton4.wav')


def zufallstoene(dummywert):		# Sampleerzeugungsfunktion fr Ton-
	global zufall			# klasse (gute Zufallszahlen)
	N = 10000
	z = zufall.randint(0, N)
	return 2.0 * z / float(N) - 1.0
	
def mein_kleiner_zufall():	# Liefert "Zufallszahlen 0 <= x < 100000
	global letzter_wert	# schlechte Funktion, hat Perioden

	a = 44079.0
	b = 151.0
	c = letzter_wert * b / a
	d = int(c % 1 * 100000)
	letzter_wert = d
	return d

# Nun die Tonfunktion
def meine_zufallstoene(dummywert):
	z = mein_kleiner_zufall()
	return (z - 50000.0) / 50000.0

def test5():
	d = 5.0
	l = 1.0
	t_zufall = Ton(dauer = d, lautstaerke = l, tonfunktion = zufallstoene)
	t_zufall.wave_speichern('ton5.wav')

def test6():
	global zufall			# Random-Klasse
	global letzter_wert		# Startwert fr eigene Zufallsfunktion

	d = 5.0				# Sekunden
	l = 1.0				# Lautstaerke
	t_mein_zufall = Ton()		# ein leerer Ton zum Anfang
	for i in range(10):
		letzter_wert = 507 + i * 16	# Startzahl setzen
		t_mein_zufall += Ton(dauer = 0.5, lautstaerke = l,
					tonfunktion = meine_zufallstoene)
	print 'und speichern'
	t_mein_zufall.wave_speichern('mein_zufallston.wav')

	seed()					# Startzahl setzen
	zufall = Random()			# Instanz initialisieren
	print 'Jetzt den zuflligen Ton'
	t_zufall = Ton(dauer = d, lautstaerke = l, tonfunktion = zufallstoene)
	print 'Jetzt als Wave speichern'
	t_zufall.wave_speichern('zufallston.wav')
	d = 1.0 / 440.0
	teilton = Ton(dauer = d, lautstaerke = 1.0)
	for i in range(teilton.samples()):
		teilton[i] = t_zufall[i]
	print 'Teilton zusammengesetzt'
	t440 = Ton()
	print 'jetzt Endton bauen'
	for i in range(440 * 5):
		if not i % 22:
			print '.',
		t440 += teilton
	t440.wave_speichern('zufallston440.wav')

def test7():
	ausgangsdatei = 'log_in_abo_0.wav'
	ergebnisdatei = 'log_in_abo_%.2f_%.2f.wav'
	t0 = Ton()
	t0.wave_laden(ausgangsdatei)
	abschwellfaktor = 0.5
#	verzoegerung = 0.1			# liefert Echo
	verzoegerung = 0.01			# liefert Hall
	neue_dauer = t0.parameter()[0] + 2.0
	samples_t0 = t0.samples()
	rate_t0 = t0.parameter()[4]
	for i in range(10):
		t = Ton(dauer = neue_dauer, tonfunktion = stille,
				rate = rate_t0)
		verschiebung = int((i + 1) * verzoegerung * rate_t0)
		samples_t = t.samples()
		print 'Ton %d:' %i ,
		print 'Verschiebung ist %d' % verschiebung,
		print 'Verzoegerung %f' % ((i + 1) * verzoegerung)
		for j in range(samples_t0):
			s0 = t0[j]
			s = t[j]
			t[j] = s0 + s
			t[j + verschiebung] += (s0 + s) * abschwellfaktor
		for j in range(samples_t0, samples_t):
			if j + verschiebung < samples_t:
				t[j + verschiebung] += t[j] * abschwellfaktor
		t.wave_speichern(ergebnisdatei %((i + 1) * verzoegerung, 
							abschwellfaktor))
	
def gerundet_256(s):
	global teiler_256

	return int(s / teiler_256 + 0.5) * teiler_256

def test8():		# Simulation von 8-Bit-Samples durch Runden in 
			# 256er-Stufen und Berechnen des Differenztones
	global teiler_256

	ausgangsdatei = 'log_in_abo_0.wav'
	t0 = Ton()
	print 'Ausgangsdatei laden'
	t0.wave_laden(ausgangsdatei)
	print 'Parameter ermitteln'
	t0dauer = t0.parameter()[0]
	t0rate = t0.parameter()[4]
	maxsample = t0.parameter()[6]
	t0_samples = t0.samples()
	teiler_256 = max(maxsample, lautstaerke_1) * 2.0 / 256.0
	print 'Zweiten Ton herstellen'
	t1 = Ton(dauer = t0dauer, rate = t0rate, tonfunktion = stille)
	s_diff = t0_samples - t1.samples()
	if s_diff > 0:		# neuer Ton hat weniger Samples (Rundungsfehler)
		for i in range(s_diff):
			t1.anfuegen(0.0)
	print 'mit Werten fllen'
	for i in range(t0_samples):
		t1[i] = gerundet_256(t0[i])
	print 'und als Wave-Datei speichern'
	t1.wave_speichern('log_in_abo_256.wav')
	print 'Differenzton erzeugen'
	t_diff = Ton(dauer = t0dauer, rate = t0rate, tonfunktion = stille)
	print 'mit Werten fllen'
	for i in range(t0_samples):
		t_diff[i] = t0[i] - t1[i]
	print 'und abspeichern'
	t_diff.wave_speichern('log_in_abo_256_diff.wav')
	print 'Fertig!'
	plotdat = open('log_in_abo.plt', 'wb')
	t0.plot(plotdat, normiert = 1)
	plotdat.close()
	plotdat = open('log_in_abo_256.plt', 'wb')
	t1.plot(plotdat, normiert = 1)
	plotdat.close()
	plotdat = open('log_in_abo_256_diff.plt', 'wb')
	t_diff.plot(plotdat, normiert = 1)
	plotdat.close()

def test9():		# Glissando in 3 Sekunden von a auf a'''
	startton = 220.0 
#	endton = 1760.0
	endton = 22050.0	# Sehr interessanter Effekt !
	gliss_dauer = 3.0
	r = 22050.0		# Samplerate

	t = Ton(dauer = gliss_dauer, rate = r, tonfunktion = stille)
	x0 = x = 2.0 * math.pi * startton / r
	incr = 2.0 * math.pi / r * (endton - startton) / (r * gliss_dauer)
	zuwachs = x0
	for i in range(t.samples()):
		w = math.sin(x)
		x += x0 +  i * incr	# So
#		zuwachs += incr		# oder so
#		x += zuwachs
		t[i] = w * lautstaerke_1
# Feste Vergleichstne 
#	t0 = Ton(dauer = 1.0, rate = r, frequenz = startton)
#	t2 =  Ton(dauer = 1.0, rate = r, frequenz = endton)
#	tgesamt = t0 + t + t2
	t.wave_speichern('gliss_%.1f_%.1f.wav'% (startton, endton))
#	plotdat = open('gliss_%.1f_%.1f.plt'% (startton, endton), 'w')
#	t.plot(plotdat)
#	plotdat.close()

#test1()
#test2()
#test3a()
#test3b()
#test3c()
#test4()
#test5()
#test6()
#test7()
#test8()
#test9()
